@@ -1,5 +1,5 @@
 # Part of the rstanarm package for estimating model parameters
-# Copyright (C) 2015, 2016 Trustees of Columbia University
+# Copyright (C) 2015, 2016, 2017 Trustees of Columbia University
 # 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -15,13 +15,15 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-#' Leave-one-out (LOO) and K-fold cross-validation
+#' Information criteria and cross-validation
 #' 
 #' For models fit using MCMC, compute approximate leave-one-out cross-validation
-#' (LOO) or, less preferably, the Widely Applicable Information Criterion (WAIC)
-#' using the \pkg{\link[=loo-package]{loo}} package. Exact \eqn{K}-fold
+#' (LOO, LOOIC) or, less preferably, the Widely Applicable Information Criterion
+#' (WAIC) using the \pkg{\link[=loo-package]{loo}} package. Exact \eqn{K}-fold 
 #' cross-validation is also available. Compare two or more models using the 
-#' \code{compare_models} function.
+#' \code{compare_models} function. \strong{Note:} these functions are not 
+#' guaranteed to work properly unless the \code{data} argument was specified 
+#' when the model was fit.
 #' 
 #' @aliases loo waic
 #'
@@ -115,7 +117,7 @@
 #' # when comparing three or more models they are ordered by expected
 #' # predictive accuracy
 #' fit3 <- stan_glm(mpg ~ ., data = mtcars)
-#' loo3 <- loo(fit3, cores = 2)
+#' loo3 <- loo(fit3, k_threshold = 0.7, cores = 2)
 #' compare_models(loo1, loo2, loo3)
 #' 
 #' # 10-fold cross-validation
@@ -140,7 +142,7 @@ loo.stanreg <- function(x, ..., k_threshold = NULL) {
   }
   loo_x <- suppressWarnings(loo.function(ll_fun(x), args = ll_args(x), ...))
   
-  bad_obs <- which(loo_x[["pareto_k"]] > k_threshold)
+  bad_obs <- loo::pareto_k_ids(loo_x, k_threshold)
   n_bad <- length(bad_obs)
   
   out <- structure(loo_x, 
@@ -167,7 +169,11 @@ loo.stanreg <- function(x, ..., k_threshold = NULL) {
     return(out)
   }
   
-  reloo(x, loo_x, obs = bad_obs)
+  reloo_out <- reloo(x, loo_x, obs = bad_obs)
+  structure(reloo_out, 
+            name = attr(out, "name"), 
+            discrete = attr(out, "discrete"),
+            yhash = attr(out, "yhash"))
 }
 
 
@@ -199,6 +205,10 @@ waic.stanreg <- function(x, ...) {
 #'   leaving out one of the \code{K} subsets. If \code{K} is equal to the total
 #'   number of observations in the data then \eqn{K}-fold cross-validation is
 #'   equivalent to exact leave-one-out cross-validation.
+#' @param save_fits If \code{TRUE}, a component 'fits' is added to the returned
+#'   object to store the cross-validated \link[=stanreg-objects]{stanreg}
+#'   objects and the indices of the omitted observations for each fold. Defaults
+#'   to \code{FALSE}.
 #'   
 #' @return \code{kfold} returns an object with has classes 'kfold' and 'loo' 
 #'   that has a similar structure as the objects returned by the \code{loo} and
@@ -214,7 +224,7 @@ waic.stanreg <- function(x, ...) {
 #'   \code{compare_models} function is also compatible with the objects returned
 #'   by \code{kfold}.
 #'   
-kfold <- function(x, K = 10) {
+kfold <- function(x, K = 10, save_fits = FALSE) {
   validate_stanreg_object(x)
   stopifnot(K > 1, K <= nobs(x))
   if (!used.sampling(x)) 
@@ -230,6 +240,7 @@ kfold <- function(x, K = 10) {
   bin <- .bincode(perm, breaks = idx, right = FALSE, include.lowest = TRUE)
   
   lppds <- list()
+  fits <- array(list(), c(K, 2), list(NULL, c("fit", "omitted")))
   for (k in 1:K) {
     message("Fitting model ", k, " out of ", K)
     omitted <- which(bin == k)
@@ -243,6 +254,8 @@ kfold <- function(x, K = 10) {
       log_lik.stanreg(fit_k, newdata = d[omitted, , drop = FALSE],
                       newx = get_x(x)[omitted, , drop = FALSE],
                       stanmat = as.matrix.stanreg(x))
+    if (save_fits) 
+      fits[k, ] <- list(fit = fit_k, omitted = omitted)
   }
   elpds <- unlist(lapply(lppds, function(x) {
     apply(x, 2, log_mean_exp)
@@ -253,6 +266,9 @@ kfold <- function(x, K = 10) {
     se_elpd_kfold = sqrt(N * var(elpds)),
     pointwise = cbind(elpd_kfold = elpds)
   )
+  if (save_fits) 
+    out$fits <- fits
+  
   structure(out, 
             class = c("kfold", "loo"), 
             K = K, 
@@ -390,13 +406,17 @@ recommend_exact_loo <- function(reason) {
 # Refit model leaving out specific observations
 #
 # @param x stanreg object
-# @param loo_x result of loo(x)
-# @param obs vector of observation indexes
+# @param loo_x the result of loo(x)
+# @param obs vector of observation indexes. the model will be refit length(obs)
+#   times, each time leaving out one of the observations specified in 'obs'.
 # @param ... unused currently
 # @param refit logical, to toggle whether refitting actually happens (only used
 #   to avoid refitting in tests)
+#
+# @return A modified version of 'loo_x'. 
+#
 reloo <- function(x, loo_x, obs, ..., refit = TRUE) {
-  stopifnot(!is.null(x$data), inherits(loo_x, "loo"))
+  stopifnot(!is.null(x$data), is.loo(loo_x))
   if (is.null(loo_x$pareto_k))
     stop("No Pareto k estimates found in 'loo' object.")
   
@@ -421,22 +441,32 @@ reloo <- function(x, loo_x, obs, ..., refit = TRUE) {
     lls[[j]] <-
       log_lik.stanreg(fit_j, newdata = d[omitted, , drop = FALSE],
                       newx = get_x(x)[omitted, , drop = FALSE],
-                      stanmat = as.matrix.stanreg(x))
+                      stanmat = as.matrix.stanreg(fit_j))
   }
   
-  # replace parts of loo_x
-  sel <- c("elpd_loo", "looic")
-  elppds <- unlist(lapply(lls, log_mean_exp))
-  loo_x$pointwise[obs, sel] <- cbind(elppds, -2 * elppds)
+  # compute elpd_{loo,j} for each of the held out observations
+  elpd_loo <- unlist(lapply(lls, log_mean_exp))
+  
+  # compute \hat{lpd}_j for each of the held out observations (using log-lik
+  # matrix from full posterior, not the leave-one-out posteriors)
+  ll_x <- log_lik(x, newdata = d[obs,, drop=FALSE])
+  hat_lpd <- apply(ll_x, 2, log_mean_exp)
+  
+  # compute effective number of parameters
+  p_loo <- hat_lpd - elpd_loo
+
+  # replace parts of the loo object with these computed quantities
+  sel <- c("elpd_loo", "p_loo", "looic")
+  loo_x$pointwise[obs, sel] <- cbind(elpd_loo, p_loo,  -2 * elpd_loo)
   loo_x[sel] <- with(loo_x, colSums(pointwise[, sel]))
   loo_x[paste0("se_", sel)] <- with(loo_x, {
     N <- nrow(pointwise)
     sqrt(N * apply(pointwise[, sel], 2, var))
   })
-  
-  # what should we do about pareto k's? for now setting them to 0
+  # what should we do about pareto k? for now setting them to 0
   loo_x$pareto_k[obs] <- 0
-  loo_x
+  
+  return(loo_x)
 }
 
 # log_mean_exp (just log_sum_exp(x) - log(length(x)))
@@ -450,18 +480,19 @@ log_mean_exp <- function(x) {
 # @param x stanreg object
 # @return data frame
 kfold_and_reloo_data <- function(x) {
-  d <- get_all_vars(formula(x), x[["data"]])
+  dat <- x[["data"]]
+  sub <- getCall(x)[["subset"]]
+  
+  d <- get_all_vars(formula(x), dat)
+  if (!is.null(sub)) {
+    keep <- eval(substitute(sub), envir = dat)
+    d <- d[keep,, drop=FALSE]
+  }
+  
   na.omit(d)
 }
 
 
-# @param x stanreg object
-family_string <- function(x) {
-  fam <- family(x)
-  if (is.character(fam)) 
-    fam else fam$family
-}
-
 # Calculate a SHA1 hash of y
 # @param x stanreg object
 # @param ... Passed to digest::sha1
