@@ -1,5 +1,5 @@
 # Part of the rstanarm package for estimating model parameters
-# Copyright (C) 2015, 2016 Trustees of Columbia University
+# Copyright (C) 2015, 2016, 2017 Trustees of Columbia University
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -179,10 +179,14 @@ posterior_predict.stanreg <- function(object, newdata = NULL, draws = NULL,
   if (is(object, "polr") && !is_scobit(object))
     ytilde <- matrix(levels(get_y(object))[ytilde], nrow(ytilde), ncol(ytilde))
   
+  if (is.null(newdata)) colnames(ytilde) <- rownames(model.frame(object))
+  else colnames(ytilde) <- rownames(newdata)
   structure(ytilde, class = c("ppd", class(ytilde)))
 }
 
 
+# internal ----------------------------------------------------------------
+
 # functions to draw from the various posterior predictive distributions
 pp_fun <- function(object) {
   suffix <- if (is(object, "polr")) "polr" else family(object)$family
