@@ -24,8 +24,10 @@
 #'   on the fit.
 #' @examples
 #' \donttest{
-#' stan_aov(yield ~ block + N*P*K, data = npk, contrasts = "contr.poly",
+#' op <- options(contrasts = c("contr.helmert", "contr.poly"))
+#' stan_aov(yield ~ block + N*P*K, data = npk,
 #'          prior = R2(0.5), seed = 12345) 
+#' options(op)
 #' }
 #'             
 stan_aov <- function(formula, data, projections = FALSE,
@@ -57,7 +59,7 @@ stan_aov <- function(formula, data, projections = FALSE,
         ## no Error term
         fit <- eval(lmcall, parent.frame())
         fit$terms <- Terms
-        fit$qr <- qr(model.matrix(Terms, data = fit$data))
+        fit$qr <- qr(model.matrix(Terms, data = fit$data, contrasts.arg = contrasts))
         R <- qr.R(fit$qr)
         beta <- extract(fit$stanfit, pars = "beta", permuted = FALSE)
         pnames <- dimnames(beta)$parameters
