@@ -1,5 +1,5 @@
 # Part of the rstanarm package for estimating model parameters
-# Copyright (C) 2013, 2014, 2015, 2016 Trustees of Columbia University
+# Copyright (C) 2013, 2014, 2015, 2016, 2017 Trustees of Columbia University
 # 
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -66,7 +66,7 @@
 #' if (!grepl("^sparc",  R.version$platform)) {
 #' ### Linear regression
 #' fit <- stan_glm(mpg / 10 ~ ., data = mtcars, QR = TRUE,
-#'                 algorithm = "fullrank") # only to make example fast enough
+#'                 algorithm = "fullrank") # for speed of example only
 #' plot(fit, prob = 0.5)
 #' plot(fit, prob = 0.5, pars = "beta")
 #' }
@@ -78,9 +78,9 @@
 #'   switch ~ dist100 + arsenic, 
 #'   data = wells, 
 #'   family = binomial(link = "logit"), 
-#'   prior = student_t(df = 7, location = 0, scale = 2.5), 
 #'   prior_intercept = normal(0, 10),
-#'   chains = 1, iter = 250 # for speed
+#'   QR = TRUE,
+#'   chains = 2, iter = 200 # for speed of example only
 #' )
 #' print(fit2)
 #' prior_summary(fit2)
@@ -96,7 +96,7 @@
 #' treatment <- gl(3,3)
 #' fit3 <- stan_glm(counts ~ outcome + treatment, family = poisson(link="log"),
 #'                  prior = normal(0, 1), prior_intercept = normal(0, 5),
-#'                  chains = 1, iter = 250) # for speed
+#'                  chains = 2, iter = 250) # for speed of example only
 #' print(fit3)
 #' 
 #' bayesplot::color_scheme_set("green")
@@ -108,16 +108,17 @@
 #' clotting <- data.frame(log_u = log(c(5,10,15,20,30,40,60,80,100)),
 #'                        lot1 = c(118,58,42,35,27,25,21,19,18),
 #'                        lot2 = c(69,35,26,21,18,16,13,12,12))
-#' fit4 <- stan_glm(lot1 ~ log_u, data = clotting, family = Gamma,
-#'                  chains = 1, iter = 250) # for speed 
+#' fit4 <- stan_glm(lot1 ~ log_u, data = clotting, family = Gamma(link="log"),
+#'                  chains = 2, iter = 300) # for speed of example only 
 #' print(fit4, digits = 2)
 #' fit5 <- update(fit4, formula = lot2 ~ log_u)
 #' 
 #' ### Negative binomial regression
 #' fit6 <- stan_glm.nb(Days ~ Sex/(Age + Eth*Lrn), data = MASS::quine, 
-#'                     link = "log", prior_aux = exponential(1/2),
-#'                     QR = TRUE, chains = 1, iter = 250) # for speed
+#'                     link = "log", prior_aux = exponential(1),
+#'                     chains = 2, iter = 200) # for speed of example only
 #' 
+#' prior_summary(fit6)
 #' bayesplot::color_scheme_set("brightblue")
 #' plot(fit6)
 #' pp_check(fit6, plotfun = "hist", nreps = 5)
@@ -140,8 +141,8 @@ stan_glm <- function(formula, family = gaussian(), data, weights, subset,
   algorithm <- match.arg(algorithm)
   family <- validate_family(family)
   validate_glm_formula(formula)
-  if (missing(data)) 
-    data <- environment(formula)
+  data <- validate_data(data, if_missing = environment(formula))
+  
   call <- match.call(expand.dots = TRUE)
   mf <- match.call(expand.dots = FALSE)
   m <- match(c("formula", "data", "subset", "weights", "na.action", "offset"), 
@@ -175,7 +176,8 @@ stan_glm <- function(formula, family = gaussian(), data, weights, subset,
   fit <- nlist(stanfit, algorithm, family, formula, data, offset, weights,
                x = X, y = Y, model = mf,  terms = mt, call, 
                na.action = attr(mf, "na.action"), 
-               contrasts = attr(X, "contrasts"))
+               contrasts = attr(X, "contrasts"), 
+               modeling_function = "stan_glm")
   out <- stanreg(fit)
   out$xlevels <- .getXlevels(mt, mf)
   if (!x) 
@@ -223,5 +225,6 @@ stan_glm.nb <- function(formula,
   mc$family <- neg_binomial_2(link = link)
   out <- eval(mc, parent.frame())
   out$call <- call
+  out$modeling_function <- "stan_glm.nb"
   return(out)
 }
